/**
 * @file   control.cpp
 * @author Jan Moravec (jan.moravec@workswell.cz)
 * @copyright (c) 2017, Workswell s.r.o., All rights reserved.
 * @date   March, 2017
 */

#include "control.h"

#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string>
#include <sys/socket.h>
#include <arpa/inet.h>

static int socketDesc = -1;
static struct sockaddr_in socketAddr;

int connectControl(char *server, int port)
{
	socketDesc = socket(AF_INET , SOCK_STREAM , 0);
	if (socketDesc == -1)
	{
		return -1;
	}

	socketAddr.sin_addr.s_addr = inet_addr(server);
	socketAddr.sin_family = AF_INET;
	socketAddr.sin_port = htons(port);

	if (connect(socketDesc , (struct sockaddr *)&socketAddr , sizeof(socketAddr)) < 0)
	{
		return -1;
	}

    return 0;
}

int closeControl(void)
{
	if(socketDesc == -1) {
		return 0;
	}

	close(socketDesc);
	socketDesc = -1;
	return 0;
}

int receiveControl(uint8_t *buf, int length, int tmout)
{
	static struct timeval tv; // Timeout
	tv.tv_sec = tmout/1000000;
	tv.tv_usec = tmout%1000000;
	setsockopt(socketDesc, SOL_SOCKET, SO_RCVTIMEO, (char *)&tv, sizeof(struct timeval));

	int bytes_num;
	if( (bytes_num = recv(socketDesc , buf, length, 0)) == -1)
	{
		return -1;
	}

	return bytes_num;
}

int sendControl(const char *mess, int length)
{
	int bytes_num;

    if( (bytes_num = send(socketDesc , mess, length, 0)) == -1 )
    {
        return -1;
    }

    return bytes_num;
}

int checkControl(void)
{
	std::string mess = "HIWS";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 200000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::cout << "checkControl(): WIRIS answer: " << ans << std::endl;

	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
}

int doCIC(void)
{
	std::string mess = "DCIC";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 3000000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);

	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
 }

int setRangeHigh(void)
{
	std::string mess = "STRH";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 7000000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);

	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
}

int setRangeLow(void)
{
	std::string mess = "STRL";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 7000000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);

	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
}

int getRange(int &lowval, int &highval)
{
	std::string mess = "GTRG";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> lowval >> highval;
	return 0;
}

int getSerialNumber(std::string &serial)
{
	std::string mess = "GWSN";
	uint8_t buff[32];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 32, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	serial.assign((char *)buff);
	return 0;
}

int getFirmwareVersion(std::string &version)
{
	std::string mess = "GFWV";
	uint8_t buff[32];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 32, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	version.assign((char *)buff);
	return 0;
}

int getDIOVersion(std::string &version)
{
	std::string mess = "GDIV";
	uint8_t buff[32];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 32, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	version.assign((char *)buff);
	return 0;
}

int getResolutionThermo(int &width, int &height)
{
	std::string mess = "GTCR";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> width >> height;
	return 0;
}

int getResolutionVisible(int &width, int &height)
{
	std::string mess = "GDCR";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> width >> height;
	return 0;
}

int getDeviceTemperature(double &temperature)
{
	std::string mess = "GDTE";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> temperature;
	return 0;
}

int getSensorTemperature(double &temperature)
{
	std::string mess = "GSTE";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> temperature;
	return 0;
}

/// 2 - 30
int setTimeToCorrection(int timeMin)
{
	std::string mess = "SCCP " + std::to_string(timeMin);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setEmissivity(double emis)
{
	std::string mess = "SEMI " + std::to_string(emis);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 1000000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int getEmissivity(double &emis)
{
	std::string mess = "GEMI";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> emis;
	return 0;
}

int setAtmosphericTemp(double temp)
{
	std::string mess = "SATE " + std::to_string(temp);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 1000000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int getAtmosphericTemp(double &temp)
{
	std::string mess = "GATE";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> temp;
	return 0;
}

int setReflectedTemp(double temp)
{
	std::string mess = "SRTE " + std::to_string(temp);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 1000000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int getReflectedTemp(double &temp)
{
	std::string mess = "GRTE";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> temp;
	return 0;
}

int getDateTime(int &year, int &month, int &day, int &hour, int &minute, int &second)
{
	std::string mess = "GDTI";
	uint8_t buff[64];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 64, 200000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> year >> month >> day >> hour >> minute >> second;
	return 0;
}

int setDateTime(int year, int month, int day, int hour, int minute, int second)
{
	std::string mess = "SDTI " + std::to_string(year) + " " + std::to_string(month) + " " + std::to_string(day) + " " + std::to_string(hour) + " " + std::to_string(minute) + " " + std::to_string(second);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 200000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int getFreeMemory(double &spaceRel)
{
	std::string mess = "GFME";
	uint8_t buff[64];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 64, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> spaceRel;
	return 0;
}

int getNumberImages(int &images)
{
	std::string mess = "GNIM";
	uint8_t buff[64];
	images = -1;

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 64, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);
	std::string type;

	stream >> images;
	return 0;
}

int getSequenceLength(int &seqS)
{
	std::string mess = "GSQL";
	uint8_t buff[64];
	seqS = -1;

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 64, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> seqS;
	return 0;
}

int captureImages(void)
{
	std::string mess = "CIMG";
	uint8_t buff[32];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 32, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int startSequence(void)
{
	std::string mess = "STSQ";
	uint8_t buff[32];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 32, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int stopSequence(void)
{
	std::string mess = "SPSQ";
	uint8_t buff[32];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 32, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int getSequence(bool &capturing)
{
	std::string mess = "GCSQ";
	uint8_t buff[32];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 32, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("YES" == ans) {
		capturing = true;
	} else {
		capturing = false;;
	}
	return 0;
}

int setCaptureRadiometric(bool capture)
{
	std::string mess = "SCRJ";
	if(capture == true) {
		mess += " ON";
	} else {
		mess += " OFF";
	}
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setCaptureScreen(bool capture)
{
	std::string mess = "SCNR";
	if(capture == true) {
		mess += " ON";
	} else {
		mess += " OFF";
	}
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setCaptureTiff(bool capture)
{
	std::string mess = "SCRT";
	if(capture == true) {
		mess += " ON";
	} else {
		mess += " OFF";
	}
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setCaptureVisible(bool capture)
{
	std::string mess = "SCVJ";
	if(capture == true) {
		mess += " ON";
	} else {
		mess += " OFF";
	}
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setVisibleScale(int scale)
{
	std::string mess = "SVIS " + std::to_string(scale);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

/*
blackred
bluered
bluered
bwiron
bwironi
bwrainbow
bwrainbowhc
bwrgb
fire
gradient
gray
iron1
natural
rainbow
rainbowhc
sepia
steps
wbrgb
 */
int setPalette(std::string palette)
{
	std::string mess = "SPAL " + palette;
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setRangeManual(bool manual)
{
	std::string mess = "SRML";
	if (manual == true) {
		mess += " ON";
	} else {
		mess += " OFF";
	}
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setRangeMinimum(double minimum)
{
	std::string mess = "SRMI " + std::to_string(minimum);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int setRangeMaximum(double maximum)
{
	std::string mess = "SRMA " + std::to_string(maximum);
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int getGPS(bool &status, double &latitude, double &longitude, double &altitude)
{
	std::string mess = "GGPS";
	uint8_t buff[64];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 64, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	std::stringstream stream(ans);

	stream >> status >> latitude >> longitude >> altitude;
	return 0;
}

int stopEthernetMode()
{
	std::string mess = "SPET";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 5000000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int shutDownWiris()
{
	std::string mess = "SHTW";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}

int rebootWiris()
{
	std::string mess = "RBTW";
	uint8_t buff[16];

	int return_value = sendControl(mess.c_str(), mess.length());
	if(return_value == -1) {
		return -1;
	}

	return_value = receiveControl(buff, 16, 100000);
	if(return_value == -1) {
		return -1;
	}
	buff[return_value] = '\0';
	std::string ans((char *)buff);
	if("OK" == ans) {
		return 0;
	} else {
		return -1;
	}
	return 0;
}
