/**
 * @file   saveImage.cpp
 * @author Jan Moravec (jan.moravec@workswell.cz)
 * @copyright (c) 2017, Workswell s.r.o., All rights reserved.
 * @date   March, 2017
 */

#include "saveImage.h"

#include <iostream>
#include <stdio.h>
#include <string>
#include <jpeglib.h>

void saveGrayJpeg(char *name, uint8_t *image_gray, int width, int height)
{
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	FILE *outfile = fopen(name, "wb");
	JSAMPROW row_pointer[1];
	// create jpeg data
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);
	jpeg_stdio_dest(&cinfo, outfile);
	// set image parameters
	cinfo.image_width = width;
	cinfo.image_height = height;
	cinfo.input_components = 1;
	cinfo.in_color_space = JCS_GRAYSCALE;
	cinfo.dct_method = JDCT_IFAST;
	// set jpeg compression parameters to default
	jpeg_set_defaults(&cinfo);
	// and then adjust quality setting
	jpeg_set_quality(&cinfo, 80, TRUE);
	// start compress
	jpeg_start_compress(&cinfo, TRUE);
	// Create the jpeg
	int row_stride = width;

	while (cinfo.next_scanline < cinfo.image_height) {
		row_pointer[0] = &image_gray[cinfo.next_scanline * row_stride];
		jpeg_write_scanlines(&cinfo, row_pointer, 1);
	}

	// finish compression
	jpeg_finish_compress(&cinfo);
	// destroy jpeg data
	jpeg_destroy_compress(&cinfo);
	// close output file
	fclose(outfile);
}

void saveVisibleJpeg(char *name, uint8_t *image_vis, int width, int height)
{
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	FILE *outfile = fopen(name, "wb");
	JSAMPROW row_pointer[1];
	// create jpeg data
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);
	jpeg_stdio_dest(&cinfo, outfile);
	// set image parameters
	cinfo.image_width = width;
	cinfo.image_height = height;
    cinfo.input_components = 3;
    cinfo.in_color_space = JCS_YCbCr;
    cinfo.dct_method = JDCT_IFAST;
	// set jpeg compression parameters to default
	jpeg_set_defaults(&cinfo);
	// and then adjust quality setting
	jpeg_set_quality(&cinfo, 80, TRUE);
	// start compress
	jpeg_start_compress(&cinfo, TRUE);

	// Create the jpeg
	uint8_t *img = new uint8_t[width * 3];
    // Feed image data
    row_pointer[0] = img;

    while (cinfo.next_scanline < cinfo.image_height) {
        unsigned i, j;
        unsigned offset = cinfo.next_scanline * cinfo.image_width * 2;

        for (i = 0, j = 0; i < cinfo.image_width * 2; i += 4, j += 6) {
            img[j + 0] = image_vis[offset + i + 0]; // Y
            img[j + 1] = image_vis[offset + i + 1]; // U
            img[j + 2] = image_vis[offset + i + 3]; // V
            img[j + 3] = image_vis[offset + i + 2]; // Y
            img[j + 4] = image_vis[offset + i + 1]; // U
            img[j + 5] = image_vis[offset + i + 3]; // V
        }

        jpeg_write_scanlines(&cinfo, row_pointer, 1);
    }

	// finish compression
	jpeg_finish_compress(&cinfo);
	// destroy jpeg data
	jpeg_destroy_compress(&cinfo);
	delete[] img;
	// close output file
	fclose(outfile);
}
