/**
 * @file   thermo.h
 * @author Jan Moravec (jan.moravec@workswell.cz)
 * @copyright (c) 2017, Workswell s.r.o., All rights reserved.
 * @date   March, 2017
 * @brief  This files contains functions for connecting and streaming the thermal images from WIRIS thermal port.
 */

#ifndef SRC_LIB_THERMO_H_
#define SRC_LIB_THERMO_H_

#include <stdint.h>

/// Connect to server - IP address and port number
int connectThermo(char *server, int port);
/// Close connection
int closeThermo(void);

/// Basic functions receiving data
int receiveThermo(uint8_t *buf, int length, int tmout);
/// Basic functions for sending
int sendThermo(const char *mess, int length);

/// Check connection, 0 if OK, -1 if error
int checkThermo(void);
/// Return thermo frame from WIRIS
int getFrameThermo(uint16_t *buf, int width, int height);
/// Return thermo frame from WIRIS, numbered method
int getFrameThermoNumbered(uint16_t *buf, int width, int height);

/// Calculate temperature K from raw pixel
double rawToTemperatureK(uint16_t pixRaw);
/// Calculate temperature C from raw pixel
double rawToTemperatureC(uint16_t pixRaw);

#endif /* SRC_LIB_THERMO_H_ */
